#! /usr/bin/wish
#add current folder for path
lappend auto_path [pwd]
#ask for BWidgets
package require BWidget

#main window configuration
wm title . "Unicorn II Configuration"
wm resizable . 0 0

#data structs
namespace eval Parms {
	set VPI 8
	set VCI 35
	set Protocol "PPPOA"
	set Encaps "LLC"
	set ActivationMode "MULTI Annex A"
	set IP "192.168.1.50"
}

namespace eval Target {
	set VPI ""
	set VCI ""
	set Protocol ""
	set Encaps ""
	set ActivationMode ""
	set IP ""
}
#check IP address format
proc isIP {str type} {
	set ipnum1 {\d|[1-9]\d|1\d\d|2[0-4]\d|25[0-5]}
	set ipnum2 {\d|[1-9]\d|1\d\d|2[0-4]\d|25[0-5]}
	set ipnum3 {\d|[1-9]\d|1\d\d|2[0-4]\d|25[0-5]}
	set ipnum4 {\d|[1-9]\d|1\d\d|2[0-4]\d|25[0-5]}
	set fullExp {^($ipnum1)\.($ipnum2)\.($ipnum3)\.($ipnum4)$}
	set partialExp {^(($ipnum1)(\.(($ipnum2)(\.(($ipnum3)(\.(($ipnum4)?)?)?)?)?)?)?)?$}
	set fullExp [subst -nocommands -nobackslashes $fullExp]
		set partialExp [subst -nocommands -nobackslashes $partialExp]
		if [string equal $type focusout] {
			if [regexp -- $fullExp $str] {
				return 1
			} else {
				return 0
			}
		} else {
			return [regexp -- $partialExp $str]
		}
}
#valibate IP address
proc CheckIP { ip } {
	set isIP "1"
	if {$ip == ""}                           then {set isIP "0"}
	if {[llength [split $ip .] ] != 4}       then {set isIP "0"}
	foreach i [split $ip .] {
		if {[string is integer $i] == 1} then {
			if {$i > 255} then {set isIP "0"}
			if {$i < 0}   then {set isIP "0"}
		} else { set isIP "0"
		}
	}

	set IPAddrRe {^(\d|[1-9]\d|1\d\d|2[0-4]\d|25[0-5])\.(\d|[1-9]\d|1\d\d|2[0-4]\d|25[0-5])\.(\d|[1-9]\d|1\d\d|2[0-4]\d|25[0-5])\.(\d|[1-9]\d|1\d\d|2[0-4]\d|25[0-5])$}

	if { $isIP  && ! [regexp -- $IPAddrRe $ip] } {
		set isIP "0"
	}

	return $isIP
}

	

#GUI design of the application
set MainFrame  [frame .mf ]
set DescFrame  [frame .mf.left ]
set InputFrame [frame .mf.right ]

pack $MainFrame -side top -expand yes -fill both
pack $DescFrame -side left -expand yes -fill both
pack $InputFrame -side right -expand yes -fill both

set lact [ label $DescFrame.l0 -text "Activation Mode:" -width 20 -justify right  ]
set lvpi [ label $DescFrame.l1 -text "VPI:" -width 20 -justify right  ]
set lvci [ label $DescFrame.l2 -text "VCI:" -width 20 -justify right ]
set lpro [ label $DescFrame.l3 -text "Protocol:" -width 20 -justify right ]
set lenc [ label $DescFrame.l4 -text "Encapsulation :" -width 20 -justify right ]
set lip  [ label $DescFrame.l5 -text "IP Address :" -width 20 -justify right ]
set bexi [ button $DescFrame.b0 -text "Exit" -width 20 -command exit ]

pack $lact -pady 1 -anchor nw
pack $lvpi -pady 1 -anchor nw
pack $lvci -pady 1 -anchor nw
pack $lpro -pady 1 -anchor nw
pack $lenc -pady 1 -anchor nw
pack $lip  -pady 1 -anchor nw
pack $bexi -pady 1 -anchor nw

set eact [ ComboBox $InputFrame.e0 -text "MULTI Annex A" -width 20 -editable no -textvariable Parms::ActivationMode]
set evpi [ entry    $InputFrame.e1 -width 20 -validate key -vcmd { string is int %P } -textvariable Parms::VPI ]
set evci [ entry    $InputFrame.e2 -width 20 -validate key -vcmd { string is int %P } -textvariable Parms::VCI]
set epro [ ComboBox $InputFrame.e3 -text "PPPoE" -width 20 -editable no -textvariable Parms::Protocol]
set eenc [ ComboBox $InputFrame.e4 -text "LLC" -width 20 -editable no -textvariable Parms::Encaps]
set eip  [ entry    $InputFrame.e5 -width 20 -textvariable Parms::IP  -validate all -vcmd "isIP %P %V" -state disabled -fg gray ]
set bapp [ button   $InputFrame.b0 -text "Apply" -width 20 -command Configure ]


pack $eact -pady 0.5 -anchor nw
pack $evpi -anchor nw
pack $evci -anchor nw
pack $epro -anchor nw
pack $eenc -anchor nw
pack $eip  -anchor nw
pack $bapp -anchor nw

$eact insert 0 "ANSI Annex A" "GLITE Annex A" "MULTI Annex A" "GDMT Annex A" "ETSI Annex B" "GDMT Annex B" "MULTI Annex B" "GDMT BDT Annex B"

$epro insert 0 "PPPoA" "PPPoE" "Bridge" "IPoATM"
$eenc insert 0 "LLC" "VC MUX"

proc StateIpEntry { } {
	global eip
	if { $Parms::Protocol == "Bridge" || $Parms::Protocol == "IPoATM" } {
		$eip configure -state normal -fg black
	} else {
		$eip configure -state disabled -fg gray
	}
	after 1000 StateIpEntry
}

proc Configure { } {
	switch $Parms::ActivationMode {
		"ANSI Annex A" {
			set Target::ActivationMode 1
		}
		"GLITE Annex A" {
			set Target::ActivationMode 2
		}
		"MULTI Annex A" {
			set Target::ActivationMode 3
		}
		"GDMT Annex A" {
			set Target::ActivationMode 4
		}
		"ETSI Annex B" {
			set Target::ActivationMode 5
		}
		"GDMT Annex B" {
			set Target::ActivationMode 6
		}
		"MULTI Annex B" {
			set Target::ActivationMode 7
		}
		"GDMT BDT Annex B" {
			set Target::ActivationMode 8
		}
		default {
			tk_messageBox -message "activation modem not defined"
			return
		}
	}

	if { $Parms::VPI == "" } {
		tk_messageBox -message "VPI not present"
		return
	} else {
		set Target::VPI $Parms::VPI
	}
	
	if { $Parms::VCI == "" } {
		tk_messageBox -message "VCI not present"
		return
	} else {
		set Target::VCI $Parms::VCI
	}
	switch $Parms::Protocol {
		"PPPoA" {
			set Target::Protocol "pppoatm"
		}
		"PPPoE" {
			set Target::Protocol "pppoe"
		}
		"Bridge" {
			set Target::Protocol "br2684"
		}
		"IPoATM" {
			set Target::Protocol "ipoatm"
		}
		default {
			tk_messageBox -message "protocol not defined"
			return
		}
	}
	switch $Parms::Encaps {
		"LLC" {
			set Target::Encaps "llc-encaps"
		}
		"VC MUX" {
			set Target::Encaps "vc-encaps"
		}
		default {
			tk_messageBox -message "Encaps not defined"
			return
		}
	}
	set Target::IP ""
	if { $Parms::Protocol == "Bridge" || $Parms::Protocol == "IPoATM" } {
		#Check if unicorn is present
		catch { exec /sbin/ifconfig dsl0 } UnicornAttached
		if [ string match "*Device not found*" $UnicornAttached ] {
			#unicorn is not connected to the system
			tk_messageBox -message "Unicorn II is not connected\n Cann't configure IP address"
			set Target::IP ""
		} else {
			#Unicorn is connected to the system
			#verify the supplied IP address
			if [ CheckIP $Parms::IP ]  {
				#valid ip address passed
				set Target::IP $Parms::IP
			} else {
				tk_messageBox -message "Invalid IP address"
				return
			}
		}
	}

	

	catch { exec /sbin/rmmod unicorn_usb_eth } err1
	catch { exec /bin/sleep 2 } err2
	catch { exec /sbin/modprobe unicorn_usb_eth ActivationMode=$Target::ActivationMode VPI=$Target::VPI VCI=$Target::VCI PROTOCOL=$Target::Protocol ENCAPS=$Target::Encaps } err3
	if { $Target::IP != "" } {
		catch { exec /sbin/ifconfig dsl0 $Target::IP } err4
	}
}
StateIpEntry 
