/*
 * File: Compiler.h
 * Created: Tue 29 Jan 2008
 * Last modified: Tue 23 Apr 2008
 * Author: Adrian Łubik
 * License: GNU LGPL v3 or newer
 */

#ifndef SCRIPTWRITER_H
#define SCRIPTWRITER_H

#include <QString>
#include <QStringList>
#include <QDebug>
#include <QSettings>
#include <QFile>
#include <QDir>
#include <QFileInfo>
#include <QThread>

#include "MyProcess.h"
#include "SystemInfo.h"
#include "FileManager.h"

/**
  * Does: Compiles and installs drivers from sources.
  */
class Compiler: public QObject
{
    Q_OBJECT
public:
    typedef struct ModemDriver
    {
        QString name;
        QString version;
        QString extracted_driver_path;
        QString compiled_driver_name;
        QString extraction_path;
        QString driver_bundle_path;
        QString compilation_arguments;
        QString installation_arguments;
        QString environment_variables;

        bool amd64_compatible;
    };

    /**
      *Class constructor.
      *@param driverIniFilePath is a path to the ini file containing informations about driver.
      */
    Compiler ( const ModemDriver driver, QThread *parent = 0 );
    /**
      *Class destructor.
      *Writes a log in /opt/ubudsl/logs/<driver_name>-<driver_version>_compiler.log that contains debug output of driver preparation and installation processes.
      */
    virtual ~Compiler();

    int  compileDriver();
    int  cleanDriver();
    void extractDriver();
    void removeDriver ();
    int  installDriver();
    void backupDriver ();
    void restoreDriver();
    void patchDriver();

private slots:
    void debugDriverCompileProcess ( QByteArray ba );
    void debugDriverRemoveProcess ( QByteArray ba );
    void debugDriverInstallProcess ( QByteArray ba );

private:
	MyProcess   *m_compileDriverProcess;
	MyProcess   *m_cleanDriverProcess;
	MyProcess   *m_installDriverProcess;
	MyProcess   *m_extractDriverProcess;
	QProcess    *m_patchProcess;
	
    QString      extractedDriverPath;
    QString      driverName;
    QString      driverVersion;
    QString      extractionPath;
    QString      driverBundlePath;
    QString      compiledDriverName;
    QStringList  compilationArguments;
    QStringList  installationArguments;
    QStringList  environmentVariables;
    QByteArray   compilerDebug; //grabbed messages of make process
    QStringList  backupList;    //list of drivers to restore if compilation failed
    QString      kernel;
    QString      backupPath;
    QDir        *backupDir;
    QStringList  backedUpModules;

signals:
    void fatalError ( const QString & );
};

#endif

