/*
 * File: FileManager.h
 * Created: Thu 31 Jan 2008
 * Last modified: Tue 23 Apr 2008
 * Author: Adrian Łubik
 * License: GNU LGPL v3 or newer
 */

#ifndef FILEMANAGER_H
#define FILEMANAGER_H

#include <QString>
#include <QStringList>
#include <QDebug>
#include <QFileInfo>
#include <QFile>
#include <QDir>


/**
  * Class used to manage files (find, copy, remove, etc.).
  */
class FileManager: public QObject
{
    Q_OBJECT
public:
    FileManager();
    virtual ~FileManager();

    void findFile ( const QString rootDir, const QString &fileName, QString &foundFile );
    void findFiles ( const QString rootDir, const QRegExp &fileNamePattern, QStringList &foundFiles );
    bool indexFiles ( const QString &rootDir );
    void copyFiles ( const QStringList &filesToCopy, const QString &destPath );
    void copyFile ( const QString &fileToCopy, const QString &destPath );
    void removeFile ( const QString &fileName );

    void         clearIndex ();
    void         clearFoundFilesList ();
    QStringList  getIndex ();

private:
    QStringList mAllFiles;
    QStringList mIndexedFiles;
    QStringList mTmpIndexedFiles;
    int         depth;
};

#endif
