/*
 * File: modemmanager.h
 * Created: Fri 08 Feb 2008
 * Last modified: Fri 29 Feb 2008
 * Author: Adrian Łubik
 * License: GNU LGPL v. 3 or newer
 */

#ifndef MODEMMANAGER_H
#define MODEMMANAGER_H

#include <QDBusReply>
#include <QDBusConnection>
#include <QDBusInterface>
#include <QDebug>
#include <QMap>
#include <QFile>
#include <QSettings>
#include <QString>
#include <QTimer>

class ModemManager: public QObject
{
    Q_OBJECT

public:
    typedef struct Modem
    {
        QString name;
        QString driver_name;
        QString driver_version;
        QMultiMap<int, int> usb_id;
    };

    ModemManager ();

    bool    getIsModemFound ();
    QString getDriverName ();
    QString getDriverVersion ();
    QString getModemName ();
    QString getUDI();
    int     getVendorID ();
    int     getProductID ();

public slots:
    void connectToDBus();

private slots:
    void deviceAdded ( const QString &udi );
    void verifyDevice();

private:
    void initModemsMap();
    bool modemFound;
    bool driverInSystem;
    int  n;
    int  vendorID;
    int  productID;

    QList<Modem>  m_modems;
    QString       driverName;
    QString       driverVersion;
    QString       modemName;
    QTimer       *deviceTimer;
    QString       lastAdded;

    Modem sagem_fast_800_e4;
    Modem sagem_fast_800;
    Modem thomson_speedtouch_330;
    Modem zte_zxdsl_852;
    Modem cxacru_based;

signals:
    void message ( const QString &msg );
    void foundModem();

};

#endif
