/*
 * File: MyProcess.h
 * Last modified: Tue 29 Jan 2008
 * Author: Ricardo Villalba
 * Note: This file was taken from the SMplayer project
 */

#ifndef MY_PROCESS_H
#define MY_PROCESS_H

#include <QProcess>

class MyProcess : public QProcess
{
    Q_OBJECT

public:
    MyProcess ( QObject * parent = 0 );

    void addArgument ( const QString & a ); //!< Add an argument

    void clearArguments();   //!< Clear the list of arguments
    QStringList arguments();  //!< Return the list of arguments

    void start();    //!< Start the process
    void start ( const QString &program, const QStringList &arg );
    bool isRunning();   //!< Return true if the process is running

signals:
    //! Emitted when there's a line available
    void lineAvailable ( QByteArray ba );

protected slots:
    void readStdOut();   //!< Called for reading from standard output
    void procFinished();  //!< Called when the process has finished

protected:
    //! Return true if it's possible to read an entire line.
    /*! @param from specifies the position to begin. */
    int canReadLine ( const QByteArray & ba, int from = 0 );
    //! Called from readStdOut() and readTmpFile() to do all the work
    void genericRead ( QByteArray buffer );

private:
    QString program;
    QStringList arg;

    QByteArray remaining_output;
};

#endif
