/*
 * File: PackageManager.h
 * Created: Wed 30 Jan 2008
 * Last modified: Sat 01 Mar 2008
 * Author: Adrian Łubik
 * License: GNU LGPL v3 or newer
 */

#ifndef PACKAGEMANAGER_H
#define PACKAGEMANAGER_H

#include <QString>
#include <QStringList>
#include <QDebug>
#include <QFile>
#include <QDir>
#include <QList>
#include <QSettings>
#include <QThread>

#include "Apt.h"
#include "Zypper.h"
#include "SystemInfo.h"
#include "FileManager.h"

/**
  * Compiles and installs drivers from sources.
  */
class PackageManager: public QObject
{
    Q_OBJECT
public:
    /**
      *Class constructor.
      *@param packageManagerIniFilePath is a path to the ini file containing informations about package manager commands and packages list.
      */
    PackageManager ( const QString &manager, QThread *parent /*= 0*/ );
    /**
      *Class destructor.
      *Writes a log in /opt/ubudsl/logs/packagemanager.log that contains debug output of package manager processes.
      */
    virtual ~PackageManager();

    int installPackages ( const QStringList &pkgs );
    int addCD ( const QString &mntPoint );
    bool getIsPackageInstalled ( const QString &pkg );
    bool getArePackagesAvailable ( const QStringList pkgs );
    QStringList getNotInstalled ( const QStringList pkgs );

private:
    /**
      *Asks packages database whether package is already installed.
      *@param package is the name of package which is being checked.
      *@return true if the package was found in system, false otherwise.
      */

    bool    packageIsInstalled;
    QString m_manager;

    Apt *apt;
    Zypper *zypper;
};

#endif
