/*
 * File: ScriptManager.h
 * Created: Sun 27 Jan 2008
 * Last modified: Wed 23 Apr 2008
 * Author: Adrian Łubik
 * License: GNU LGPL v3 or newer
 */

#ifndef SCRIPTMANAGER_H
#define SCRIPTMANAGER_H

#include <QString>
#include <QDebug>
#include <QFile>
#include <QFileInfo>
#include <QDir>
#include <QByteArray>

#include "SystemInfo.h"

/**
  * Does: Writes vary scripts needed to establish connection.
  */
class ScriptManager: public QObject
{
    Q_OBJECT

public:
    ScriptManager();
    virtual ~ScriptManager();
    /**
      * Sets correct permissions of the script files.
      * If Everything went all right returns true;
      */
    void setScriptsPermissions();
    /**
      *Writes a script that is used to start the UbuDSL daemon.
      *It is used by unix init system.
      */
    //bool writeAutostartScript();
    /**
      *Writes $PATH_TO_PPP_DIRECTORY/options script that has
      *some options used by all the peers scripts.
      */
    bool writeOptionsScript();
    /**
      *Writes pppd's script used to connecting by the PPPoA protocol.
      */
    bool writePPPoAScript ( const unsigned short &VPI, const unsigned short &VCI, const QString &serviceLogin );
    /**
      *Writes pppd's script used to connecting by the PPPoE protocol.
      */
    bool writePPPoEScript ( const QString &serviceLogin );
    /**
      *Writes pap- and chap-secrets files that contain login and password to the service.
      */
    bool writeSecretScripts ( const QString &serviceLogin, const QString &servicePassword );
    /**
      *Writes pppd's script used to connecting using Unicorn-II based modems (ZTE ZXDSL 852).
      */
    bool writeUnicornScript ( const QString &serviceLogin );

    /**
      *Removes all the lines beggining from "sourceStr" in the file and adds one line containing "destStr" at the end of the file.
      *This function is used mostly with module's blacklist file.
      *@param fileName is the name of the file that will be open to replace strings.
      *@param sourceStr is a string that function will search at begin of each line and remove matching.
      *@param destStr is a string that will be written at the end of the file.
      */
    void replaceInFile ( const QString &fileName, const QString &sourceStr, const QString &destStr );

    /**
      *Removes all the matching lines (do not invoke it with '\n' at the end of the lineToRemove parameter - it containst this already in the function body).
      */
    void removeFromFile ( const QString &fileName, const QString &lineToRemove );

    /**
      *Adds given line to the file (do not invoke it with '\n' at the end of the lineToRemove parameter - it containst this already in the function body).
      *If such line is already med in the file, this function does nothing.
      */
    void writeToFile ( const QString &fileName, const QString &lineToWrite );
    
    void writeUnicornOption( const QString &optionsFile, int &vpi, int &vci, int &protocol );
    
    void addDaemonToStart( const int &number = 90 );

private:
	QString getRcDir();
    //Parser  *parser;
    QFile   *m_scriptFile;
    QString  autostartScript;
    QString  pppScript;
    QString  papSecretsScript;
    QString  chapSecretsScript;
    QString  unicornScript;
    QString  optionsScript;
};

#endif
