/*
 * File: ServiceParser.h
 * Created: Thu 07 Feb 2008
 * Last modified: Wed 23 Apr 2008
 * Author: Adrian Łubik
 * License: GNU LGPL v. 3 or newer
 */

#ifndef SERVICEPARSER_H
#define SERVICEPARSER_H

#include <QDebug>
#include <QDir>
#include <QDomDocument>
#include <QDomElement>
#include <QDomNodeList>
#include <QFile>
#include <QString>
#include <QWidget>
#include <QMap>

class ServiceParser: public QObject
{
    Q_OBJECT
public:
    ServiceParser ( const QString &xmlFile );
    void getCountries ();

public slots:
    void requestServiceData ( const QString &service );
    void getServicesForCountry ( const QString &ccode );

private:
    int n;
    QDomDocument domdocument;
    QDomElement rootElement;
    QDomElement countryElement;
    QDomElement serviceElement;
    QDomElement vpiElement;
    QDomElement vciElement;
    QDomElement protocolElement;
    QDomElement dnsServer1Element;
    QDomElement dnsServer2Element;
    QDomNodeList countryElements;
    QDomNodeList serviceElements;
    QDomNodeList mainElements;

    QString attributeCode;
    QString currentCountry;
    QString protocol;

    QStringList dnsServers;

    QFile ispListFile;

    int VPI;
    int VCI;

signals:
    void countriesMap ( const QMap<int, QString> &cMap );
    void servicesMap ( const QMap<int, QString> &sMap );
    void foundCountry ( const QString  &str );
    void foundService ( const QString  &str );
    void sendServiceData ( const int &VPI,
                           const int &VCI,
                           const QString  &protocol,
                           const QStringList &dnsServers );

};

#endif
