/*
 * File: Zypper.h
 * Created: Apr 2008
 * Last modified: Wed 23 Apr 2008
 * Author: Adrian Łubik
 * License: GNU LGPL v3 or newer
 */

#include <QDebug>
#include <QString>
#include <QStringList>
#include <QByteArray>
#include <QRegExp>
#include <QProcess>
#include <QThread>

#include "MyProcess.h"
#include "Rpm.h"

#ifndef _ZYPPER_H_
#define _ZYPPER_H_

struct ZypperRepo
{
    int number;
    bool enabled;
    bool refresh;
    QString type;
    QString alias;
    QString name;
};

class Zypper: public Rpm
{
    Q_OBJECT
public:
    Zypper( QThread *parent = 0 );
    virtual ~Zypper();

    int  install ( const QStringList &pkgs );
    int  addRepo ( const QString &path, const QString &alias );
    bool enableRepo ( QString alias, bool enable );
    bool enableRepo ( const QString &alias ) { return enableRepo ( alias, true ); }
    bool disableRepo ( const QString &alias ) { return enableRepo ( alias, false ); }
    bool getArePackagesAvailable ( const QStringList &pkgs );
    QList<ZypperRepo> getRepos();

private slots:
    void debugProcess ( QByteArray ba );
    void debugZyppRepos ( QByteArray ba );

private:
    QByteArray dbg;
    MyProcess *m_installProcess;
    MyProcess *m_zypperProcess;
    QProcess  *m_addRepoProcess;
    QProcess  *m_modifyRepoProcess;
    QProcess  *m_getAvailableProcess;

    QList<ZypperRepo> repos;
    QRegExp repoRegExp;
    QRegExp processing;
    QThread *m_parentThread;
};

#endif
